##################################################################################
##################################################################################
# SEM-4_EXERCISE-1: STRUCTURAL EQUATION MIXTURE MODELING
# TOPICS: DIRECT AND INDIRECT APPLICATIONS IN R/NLSEM AND MPLUS 
#
# THIS SYNTAX INCLUDES ALL ANALYSES FOR THE EXERCISES
# EXTERNAL FILES:
# MPLUS INPUTS
# gcmm2.inp, gcmm3.inp, gcmm4.inp, gcmm2_nopsi.inp
# gcmm2b.inp, gcmm3b.inp, gcmm4b.inp, gcmm2b_nopsi.inp
# hgcm1.inp
# semm2_pisa.inp, semm3_pisa.inp, semm4_pisa.inp
# FACTOR SCORES:
# fscores_gcmm2.dat, fscores_hgcm1.dat
# MPLUS OUTPUTS HAVE SAME NAMES BUT .out ENDING
##################################################################################
##################################################################################

##################################################
##################################################
# EXERCISE 1. growth curve mixture modeling
# A direct application with CD4 cell counts
##################################################
##################################################
library(MplusAutomation)
library(plotSEMM)


#library(scatterplot3d)
#library(nlsem)


dat1 <- read.table("cd4.dat",header=T)
head(dat1)

# illustration of data
par(mfrow=c(2,2))
plot(density(dat1$CD1,na.rm=T))
plot(density(dat1$CD2,na.rm=T))
plot(density(dat1$CD3,na.rm=T))
plot(density(dat1$CD4,na.rm=T))

##################################################
# 1a. GCMM without covariates
# RUN models with 2 to 4 classes
##################################################

# read fit indices
setwd("C:/holger/Workshops/nonlinear/exercises/day2/mplus_version")
# with separate variance estimates
gcmm2 <- extractModelSummaries("gcmm2.out")
gcmm3 <- extractModelSummaries("gcmm3.out")
gcmm4 <- extractModelSummaries("gcmm4.out")

c(gcmm2$BIC,gcmm3$BIC,gcmm4$BIC)
c(gcmm2$AIC,gcmm3$AIC,gcmm4$AIC)
# => 2 classes (AIC is liberal)

# read parameter estimates
gcmm2.est <- extractModelParameters("gcmm2.out")
gcmm2.est$unstandardized
# => negative residual variance in class 2

# We rerun the model (and also save factor scores)
gcmm2c <- extractModelSummaries("gcmm2_nopsi.out")
c(gcmm2$BIC,gcmm2c$BIC)
# => fixed residual variance in class 2 is ok
# i.e. there are  no interindividual difference in the development in this class

###########################
# illustration of results
# 1. growth factors
# 2. trajectories
###########################
gcmm2s.est <- extractModelParameters("gcmm2_nopsi.out")

muI  <- gcmm2s.est$unstandardized[c(10,31),3]
muS  <- gcmm2s.est$unstandardized[c(11,32),3]
phI  <- gcmm2s.est$unstandardized[c(16,37),3]
phS  <- gcmm2s.est$unstandardized[c(17,38),3]
phIS <- gcmm2s.est$unstandardized[c(9,30),3]

# probability for classes
muc <- gcmm2s.est$unstandardized[43,3]
pic <- exp(c(muc,0))/sum(exp(c(muc,0)))

################
# illustration 1
# model-implied distribution of intercept and slope
################
par(mfrow=c(1,2))
curve(pic[1]*dnorm(x=x,mean=muI[1],sd=sqrt(phI[1])),xlim=c(-3,20),ylim=c(0,.25),
      col="red",xlab="intercept",ylab="density")
curve(pic[2]*dnorm(x=x,mean=muI[2],sd=sqrt(phI[2])),xlim=c(-3,20),ylim=c(0,.25),
      col="blue",xlab="",ylab="",add=T)

curve(pic[1]*dnorm(x=x,mean=muS[1],sd=sqrt(phS[1])),xlim=c(-4,3),ylim=c(0,.25),
      col="red",xlab="intercept",ylab="density")
abline(v=muS[2],col="blue")

# factor scores
gcmm2c.fs <- read.table("fscores_gcmm2.dat",header=F,na.strings="*",
                        col.names = c("CD1","CD2","CD3","CD4",
                                      "int","slo","c_int","c_slo",
                                      "cprob1","cprob2","cw"))

################
# illustration 2
# univariate distributions
################
hist(gcmm2c.fs$int,freq=F)
curve(pic[1]*dnorm(x=x,mean=muI[1],sd=sqrt(phI[1])),xlim=c(-3,20),ylim=c(0,.25),
      col="red",xlab="",ylab="",add=T)
curve(pic[2]*dnorm(x=x,mean=muI[2],sd=sqrt(phI[2])),xlim=c(-3,20),ylim=c(0,.25),
      col="blue",xlab="",ylab="",add=T)

hist(gcmm2c.fs$slo,freq=F)
curve(pic[1]*dnorm(x=x,mean=muS[1],sd=sqrt(phS[1])),xlim=c(-4,3),ylim=c(0,.25),
      col="red",xlab="",ylab="",add=T)
abline(v=muS[2],col="blue")

################
# illustration 3
# bivariate distributions
################
colo <- c("red","blue")
plot(gcmm2c.fs$int,gcmm2c.fs$slo,col=colo[gcmm2c.fs$cw])

# regression coefficients (from means and covariances)
b1IS <- phIS/phI
b0IS <- muS-b1IS*muI

abline(b0IS[1],b1IS[1],col="red")
abline(b0IS[2],b1IS[2],col="blue")

# class2 is the class of ill patients with no change over time
# class1 is the class of healthy patients at t1 
# their CD4 cell count changes negatively over time 
# but with large interindividual differences


################
# illustration 4
# now we plot the trajectories for both groups
################
# empty plot 
plot(0,0,col="white",xlim=c(0,4),ylim=c(0,30),xlab="time point",ylab="CD4")

# average
abline(muI[1],muS[1],col="red")
abline(muI[2],muS[2],col="blue")

# individual slopes using the factor scores
N <- dim(gcmm2c.fs)[1]
samp <- sample(N,100)
for(i in samp){
  abline(gcmm2c.fs$int[i],gcmm2c.fs$slo[i],col=colo[gcmm2c.fs$cw[i]],lty=2)
}



##################################################
# 1b. GCMM with covariates
# RUN models with 2 to 4 classes
##################################################
gcmm2cov <- extractModelSummaries("gcmm2b.out")
gcmm3cov <- extractModelSummaries("gcmm3b.out")
gcmm4cov <- extractModelSummaries("gcmm4b.out")

c(gcmm2cov$BIC,gcmm3cov$BIC,gcmm4cov$BIC)
c(gcmm2cov$AIC,gcmm3cov$AIC,gcmm4cov$AIC)
# => 2 classes
# Additionally: investigate the Mplus files (groups are empty for c=3,4)
#               slope variance again negative in one class (c=2)
gcmm2cov.est <- extractModelParameters("gcmm2b_nopsi.out")
gcmm2cov.est$unstandardized

##################################################
# 1c. HGCM 
# THIS model uses a continuous heteroskedastic function
# for the slope variance 
##################################################

hgcm.fs <- read.table("fscores_hgcm1.dat",header=F,na.strings="*",
                        col.names = c("CD1","CD2","CD3","CD4",
                                      "zeta2","zeta2se","int","intse","slo","slose",
                                      "ga1","ga1se"))

par(mfrow=c(1,2))
plot(hgcm.fs$int,hgcm.fs$slo,main="HGCM")
plot(gcmm2c.fs$int,gcmm2c.fs$slo,col=colo[gcmm2c.fs$cw],main="GCMM")


##################################################
##################################################
# EXERCISE 2: SEMM with indirect application
##################################################
##################################################
library(moments)
library(MVN)
library(MASS)

dat <- read.table("pisa_sem.dat",header=T)

######################################
# descriptives (cf. sem-3_exercise-1)
######################################
online   <- apply(dat[,1:2],1,mean)
attitude <- apply(dat[,3:4],1,mean)
reading  <- apply(dat[,5:6],1,mean)

# univariate histograms
par(mfrow=c(1,3))
hist(online,freq=F)
curve(dnorm(x=x,mean=mean(online),sd=sd(online)),
      from=min(online),to=max(online),add=T)

hist(attitude,freq=F)
curve(dnorm(x=x,mean=mean(attitude),sd=sd(attitude)),
      from=min(attitude),to=max(attitude),add=T)

hist(reading,freq=F)
curve(dnorm(x=x,mean=mean(reading),sd=sd(reading)),from=min(reading),to=max(reading),add=T)

# bivariate scatterplot with loess approximation
par(mfrow=c(1,2))
scatter.smooth(online,reading)
scatter.smooth(attitude,reading)

# descriptive statistics
round(apply(dat,2,mean),3)
round(apply(dat,2,var),3)
round(apply(dat,2,skewness),3)
round(apply(dat,2,kurtosis),3)-3

# multivariate nonnormality
mardiaTest(dat[,1:4],qqplot=T) #iv

# predictor variables: bivariate distribution
plotmv1 <- kde2d(online, attitude, n = 50)
image(plotmv1)
contour(plotmv1, add = T)

######################################
# results from mixture modeling
######################################
# decide on number of classes
semm2 <- extractModelSummaries("semm2_pisa.out")
semm3 <- extractModelSummaries("semm3_pisa.out")
semm4 <- extractModelSummaries("semm4_pisa.out")

c(semm2$BIC,semm3$BIC,semm4$BIC)
c(semm2$AIC,semm3$AIC,semm4$AIC)
# => 2 or 3 classes (AIC is liberal and might enhance approximation here)
# Additionally: investigate the Mplus files (problem for c=4)

# extract model parameters
# FOR either 2 or 3 classes (same name for simpler code below)
semm2.est <- extractModelParameters("semm2_pisa.out")
semm2.est <- extractModelParameters("semm3_pisa.out")

# in principle, one can calculate the relationships by hand
# however, we use the plotSEMM package here
library(plotSEMM)

# 1. extract parameters nessecary for plot
# documentation provides information on that
# pi vector: K marginal class probabilities.
muc <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="Means"&
                                  semm2.est$unstandardized$param=="CW#1"|
                                  semm2.est$unstandardized$param=="CW#2",3]
pi1 <- exp(c(muc,0))/sum(exp(c(muc,0)))

# alpha1 Vector: K means of the latent predictor.
kap1 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="Means"&
                                   semm2.est$unstandardized$param=="ON",3]
kap2 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="Means"&
                                   semm2.est$unstandardized$param=="ATT",3]

# alpha2 Vector: K inercepts slopes from the within-class regression of the latent outcome on the latent predictor.
alp1 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="Intercepts"&
                                   semm2.est$unstandardized$param=="READ",3]

# beta21 Vector: K slopes from the within-class regression of the latent outcome on the latent predictor.
b1 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="READ.ON"&
                                   semm2.est$unstandardized$param=="ON",3]
b2 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="READ.ON"&
                                 semm2.est$unstandardized$param=="ATT",3]

# psi11	Vector: K within-class variances of the latent predictor.
ph1 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="Variances"&
                                   semm2.est$unstandardized$param=="ON",3]
ph2 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="Variances"&
                                  semm2.est$unstandardized$param=="ATT",3]

# psi22	Vector: K within-class variances of the latent outcome. 
ps1 <- semm2.est$unstandardized[semm2.est$unstandardized$paramHeader=="Residual.Variances"&
                                  semm2.est$unstandardized$param=="READ",3]


# online
semmdat1 <- plotSEMM_setup(pi=pi1, alpha1=kap1, alpha2=alp1, 
                           beta21=b1, psi11=ph1, psi22=ps1, points = 50)
# attitude
semmdat2 <- plotSEMM_setup(pi=pi1, alpha1=kap2, alpha2=alp1, 
                           beta21=b2, psi11=ph2, psi22=ps1, points = 50)

# Generates a plot which expresses the mixing probabilities 
# for each latent class conditioned on the latent predictor. 
par(mfrow=c(1,1))
plotSEMM_probability(semmdat1,EtaName="Online")
plotSEMM_probability(semmdat2,EtaName="Attitude")

plotSEMM_contour(semmdat1,EtaN1 = "online", EtaN2 = "readings skills",classinfo = T)
plotSEMM_contour(semmdat2,EtaN1 = "attitude", EtaN2 = "readings skills",classinfo = T)






